#!/usr/bin/perl -T

# Copyleft  1999,2000 Michiel Dethmers. All rights reversed.
# Document maintained at http://www.quipu.co.uk/SWISH-E/AutoSwish/.
# Version 1.0 21/01/2000.
#
# Originally based on lookup.cgi by Bas Meijer <bas@antraciet.nl>
#
# This program and library is free software; you can redistribute it and/or
# modify it under the terms of the GNU (Library) General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#* GNU (Library) General Public License for more details.
#
# You should have received a copy of the GNU (Library) General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#---------------------------------------------------------
#
# Purpose: This script is an example of a search script that can be used
# with AutoSwish. It will use the configuration files created by AutoSwish and
# show them to the user, to select from. 

#- Configurable Variables ---------------------------------------------------#

# set some vars to secure path
$ENV{BASH_ENV} = ""; # for running from command line
$ENV{'SCRIPT_FILENAME'} =~ /(.*)\/.*/;
$SCRIPT_ROOT = $1;
$ENV{'PATH'} = "/usr/local/bin:/usr/bin";

use CGI qw(:cgi-lib);
require "$SCRIPT_ROOT/config.pl";
require "$SCRIPT_ROOT/lib.pl";

# possible replacements for server mappings, if some URL do not directly correspond to file structures
%replace = (
	'/usr/doc' => '/docs',
	'/home/httpd/html' => '/apache'
);

$helpurl = '/SWISH-E/help.html';

#- Standard Stuff you won't need to edit -------------------------------------#

my (@query,$query,$index,$results);
my ($year,$month,$day)	= (localtime())[5,4,3];

#- Main Program -------------------------------------------------------------#

ReadParse;

@config_files = &get_config_files($CONFIGDIR);
# untaint form values
if ($in{page} =~ /(\d+)/) {
  $page = $1 || 1;
} else {
  $page = 1;
}
$in{numpp} =~ /(\d+)/;
$numpp = $1 || 10;
$indexselect = &makeselect;
$numppselect = &makenumppselect;

unless ($in{'search'} =~ /([\)\(\*\s\w]+)/) {
  &StartPage;
}
$query = $1;

$query =~ s/\s+/ and /g;
$results = "0"; # to be changed later
unless ($in{configfile} =~ /([\w\.]+)/) {
  &StartPage;
}
$configfile = $1;

# make sure the index given is a valid one 
$index = &getconfigitem("IndexFile","$CONFIGDIR/$configfile");
# untaint index
if ($index =~ /([\w\.\/\-]+)/) {
  $index = $1;
}
&StartPage if !$index;

if ($query) {
   &search_parse;
   &output;
} else {
  &StartPage;
}

#- Subroutines ---------------------------------------------------------------#

# Subroutine for constructing the Swish-E search request and formatting the results.

sub search_parse {
	
	$count=0;
	
	# Go via forked child to avoid shell escapes, report errors of the child
	open(SWISH, "$SWISHDIR/swish-e -w $query -m $results -f $index|");
	while (<SWISH>) {
	   chop;
     my ($stringone, $title, $filesize) = split(/\"/, $_);
     
     next if $title =~ /Untitled Document/i;
     	
	   if ($_ eq "err: no results") 
	   {&search_error("Your Search in <i>'$section'</i> for <strong>$query</strong> returned <B>no</B> items");}
	
	   if ($_ eq "err: could not open index file") 
	   {&search_error("Could not open SWISH Index File $index");}
	   
	   if ($_ eq "err: no search words specified") 
	   {&search_error("Please Enter at least one Search Word");}
	
	   if ($_ eq "err: a word is too common") 
	   {&search_error("One of your search terms is too common, please try again");}
	  
	   if  (/err: (.*)/) 
	   {&search_error("Swish error: $1");}	
		 

	   next if /^\D/;
	   $count++;
	   push(@results, $_);
	
	}
	close SWISH;
}

# generate the pulldown menu with the sections of the archive
sub makeselect {
	$html = '<SELECT NAME="configfile">';
	foreach $config (@config_files){
		if ($config->{filename} eq $in{'configfile'}) {
			$html .= qq|<OPTION SELECTED VALUE="$config->{filename}">$config->{description}|;
			$section = $config->{description};
		} else {
			$html .= qq|<OPTION VALUE="$config->{filename}">$config->{description}|;
		}
	}
	$html .= '</SELECT>';
	return $html;
}

sub makenumppselect {
	$html = '<SELECT NAME="numpp">';
  foreach (10,20,30,40) {
    $html .= "<option selected>$_" if $_ eq $numpp;
    $html .= "<option>$_" if $_ ne $numpp;
  }
	$html .= '</SELECT>';
	return $html;
}

sub output{
	my ($out,$url,$stringone,$title,$filesize,$rank,$file,$url,$label);
	print &searchdoc_header("Search Results");
  $num = @results;
  
  my $start = ($page -1) * $numpp + 1;
  $start = 1 if $start < 0;
  my $end = $start + $numpp - 1;
  $end = $num if $end > $num;
  
  $nump = int($num / $numpp) +1;
  
  my $display = "Listing Page <b>$page</b> of $nump, document $start to $end";
  my $pagelist = " | ";
  for ($i = 1; $i<=$nump;$i++) {
    my $url = "$ENV{SCRIPT_NAME}?page=$i&numpp=$numpp&configfile=$configfile&search=$in{search}";
    $url =~ s/ /+/g;
    $pagelist .= "<a href=\"$url\">Page $i</a> | " if $i != $page;
    $pagelist .= "Page $i | " if $i == $page;
  }
  $pagelist .= " ";
	
	print <<EOT;
	<TABLE BORDER="0" WIDTH=550>
	<TR><TD>Your Search in <i>'$section'</i> for <strong>$query</strong> returned $num items, listed in order of computed relevance. $display
	</P>
	</TR>
	<TR><TD>$pagelist</TR>
  <TR><TD><HR NOSHADE WIDTH="100%" SIZE="4"></TR>
EOT

  my $cnt = 0;
	foreach (@results) {
    $cnt++;
    next if $cnt < $start;
     
    ($stringone, $title, $filesize) = split(/\"/, $_);
	  ($rank, $file) = split(/ /, $stringone);
	  $url = $file;
	  $label = $file;
     
    $summary = &getSummary($file);

    # translate pathnames to urls according to %replace rules
    foreach $path (keys %replace){
   	  $url =~ s/$path/$replace{$path}/;
    }
    $line = qq|<TR><TD><B>$cnt. <A HREF=\"$url\">$label</A></B></TD></TR><TR><TD><B>$title</B> <I>Score: $rank</I> <font size=-1>[$filesize bytes]</font></TD><TR><TR><TD><i>Summary:</i> $summary</TD>\n<TR><TD><HR NOSHADE WIDTH="100%"></TD></TR>|;         
    print $line;
    last if $cnt >= $end;
	}
 	print <<EOT;
  <TR><TD>$pagelist</TR>
  <TR><TD><HR NOSHADE WIDTH="100%" SIZE="4"></TR>
  </TABLE>
EOT
	print &searchdoc_trailer;	
}

#Subroutine for print a generic HTML header.

sub searchdoc_header {

$document_title = $_[0];

return <<HTML_Header;
Content-type: text/html

<HTML>
<HEAD>
<TITLE>$document_title</TITLE>
<META NAME="Keywords" CONTENT="Search engine, search, query, index">
</HEAD>
$BODYTAG
<BLOCKQUOTE>
<TABLE BORDER="0" WIDTH="550">
<TR><TD><FONT SIZE="5">Searchengine</FONT></TD></TR>
<TR><TD>Search (sections of) this website using keywords. You can combine keywords with
<b>and</b>, <b>or</b> &amp; <b>not</b>, you can group clauses with parentheses <b>(</b> <b>)</b>.
Use <b>*</b> as an end of word wildcard. <A HREF="$helpurl">More Help</A>.
<FORM method="POST">
<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0" ALIGN="left">
	<TR VALIGN="top">
		<TD><B>Keywords</B>:</TD><TD colspan=2>
		<INPUT TYPE="TEXT" NAME="search" SIZE="35" VALUE="$query">
		<INPUT TYPE="HIDDEN" name="results" value="0"></TD>
	</TR>
	<TR VALIGN="top">
		<TD><B>in section</B>: </TD><TD>$indexselect</TD><TD> Show $numppselect Results per page</TD>
	</TR>
	<TR VALIGN="top">
		<TD COLSPAN=2 ALIGN=CENTER><INPUT TYPE="SUBMIT" VALUE="Start Search"></TD>
	</TR>
</TABLE>
</FORM>
</TD></TR>
<TR><TD COLSPAN=2>
<HR NOSHADE WIDTH="100%" SIZE="4">
</TD></TR>
</TABLE>
HTML_Header
}

# Subroutine for printing a generic HTML trailer.

sub searchdoc_trailer {
  return <<HTML_Trailer;

<TABLE BORDER="0" WIDTH="550"><tr><td align=center>
<a href="http://sunsite.berkeley.edu/SWISH-E/"><P ALIGN=CENTER><IMG SRC="$BANNER" border=0></a>
<HR>
<P ALIGN=CENTER>
Problems, Questions, Comments?<BR>
Contact the <A HREF="mailto: $WEBMAN">Web Manager</A>.
</td></tr></table>
</BODY>
</HTML>

HTML_Trailer
  exit;
}

#Subroutine for printing error messages.

sub search_error {
 $error_message = $_[0];
 print &searchdoc_header("Swish-E Search Error");
 print qq|<P>\n$error_message</P>\n|;
 print &searchdoc_trailer;      
 exit;
}

sub web_die {
  local($msg);
  print "Content-type: text/plain\n\n";
  print "Error: $msg";
  exit;
}

#Subroutine for simply displaying the input form.

sub StartPage {
   print &searchdoc_header("Swish-E Search");
   print &searchdoc_trailer;
   exit;
}

# Subroutine to retrieve the first few lines from a document.
# This does assume all documents are based in the HTML root
# and may therefore fail in some cases.

sub getSummary {
  my $res = "";
  ($doc = $_[0]) =~ s/http:\/\/[^\/]+/$WEBDIR/;
  $doc =~ s/\s//g;
  open F, $doc;
  $commenton = 0;
  while (<F>) {
    # parse out multi line comments
    if ((/<!--/ && $_ !~ /-->/) || (/-->/ && $_ !~ /<!--/)) {
      $commenton = !$commenton;
    }
    s/<[^>]+>//gs;
    $res .= $_ if !$commenton && $_ !~ /^\s+$/;
    $res =~ s/-->//g;
  }
  $res =~ s/\r//g;
  # put all text on one line and take out any <text> pairs
  # using the libwww modules would be better, but I'm trying to 
  # create as little dependencies as possible.
  $res =~ s/\n/###/g;
  $res =~ s/<[^>]+>//gs;
  $res =~ s/###/\n/g;
  close(F);
  substr($res,0,300) . "....";
}
